part of tecfy_core_client_package;

class BasePage extends StatelessWidget {
  /// title string
  final String title;

  /// code used to select the menu item
  final String code;

  /// action list
  final List<Widget>? appBarActions;

  /// app bar mottom
  final Widget? appBarBottom;

  /// body
  final Widget child;

  /// floatingActionButtonWidget
  final FloatingActionButton? floatingActionButtonWidget;

  /// is it the main page
  final bool isHome;

  /// is mobile then show a small screen even on large screen
  /// can used to popup
  final bool isMobile;
  BasePage(BuildContext context, this.title, this.code,
      {this.appBarActions,
      this.appBarBottom,
      required this.isMobile,
      required this.child,
      this.isHome = false,
      this.floatingActionButtonWidget});

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        // key: TecfyCoreApp.appGlobalPagekey,
        appBar: isMobile
            ? _getAppBar(
                context, title, appBarActions, appBarBottom as TabBar?, isHome)
            : null,
        drawer: //!Navigator.of(context).canPop() &&
            //isMobile ?
            MainDrawer2(title),
        //: null,
        floatingActionButton: isMobile ? floatingActionButtonWidget : null,
        body: _buildWidget(context, child, title, appBarActions, appBarBottom));
  }

  Widget _buildWidget(
      context, child, title, List<Widget?>? appBarActions, appBarBottom) {
    return isMobile
        ? child
        : Row(
            mainAxisAlignment: MainAxisAlignment.start,
            crossAxisAlignment: CrossAxisAlignment.start,
            children: <Widget>[
              Container(
                  child: MainDrawer2(title), width: ThemeService.menuWidth),
              SizedBox(width: 15),
              Expanded(
                  child: Column(children: <Widget>[
                SizedBox(height: MediaQuery.of(context).padding.top),
                ToolBarComponenet(),
                //Container(width: MediaQuery.of(context).size.width - 200, height: 100, child: _getAppBar(context,title,appBarActions,appBarBottom)),
                Expanded(
                    child: Card(
                        elevation: 10,
                        margin: EdgeInsets.all(10),
                        child: Column(
                          children: <Widget>[
                            Row(
                              children: <Widget>[
                                Navigator.canPop(context)
                                    ? TitlePathComponenet(code, title)
                                    : Container(),
                                Spacer(),
                                appBarActions == null
                                    ? Container()
                                    : Row(
                                        children: appBarActions
                                            .where((e) => e != null)
                                            .toList()
                                            .cast<Widget>()),
                                SizedBox(width: 5)
                              ],
                            ),
                            appBarBottom ?? Container(),
                            Expanded(child: child)
                          ],
                        ))),
              ]))
            ],
          );
  }

  PreferredSizeWidget _getAppBar(context, String title,
      List<Widget>? appBarActions, PreferredSizeWidget? bottom, bool isHome) {
    return MainAppBar(
        context: context,
        backgroundColor: Colors.transparent,
        isHome: isHome,
        title: AppLocale.translate(title),
        actions: isMobile || bottom != null ? appBarActions : null,
        bottom: bottom ?? _getActionsAsBottom(context, appBarActions));
  }

  PreferredSizeWidget? _getActionsAsBottom(context, appBarActions) {
    if (isMobile) return null;
    return PreferredSize(
        preferredSize: Size(100, 100),
        child: Row(
          children: appBarActions,
          mainAxisAlignment: MainAxisAlignment.end,
        ));
    // return PreferredSize(
    //   preferredSize: Size(LayoutService.getWidth(context),100) ,
    //   child: Align(alignment: Alignment.centerRight,child:Row(children:appBarActions, mainAxisAlignment: MainAxisAlignment.end,))
    // );
  }
}
