part of tecfy_core_client_package;

class EntityListTileComponent extends StatefulWidget {
  final Entity entity;
  final Map<String, dynamic> data;
  Property? titleField;
  Property? subTitleField;
  Property? tailField;
  final void Function(Map<String, dynamic>)? onExpandItem;
  final void Function(Map<String, dynamic>?)? onClick;
  final void Function(Map<String, dynamic>?)? onLongClick;

  EntityListTileComponent(this.entity, this.data,
      {this.onClick, this.onExpandItem, this.onLongClick}) {
    // TODO: move these properties to [Entity] class to save calling them everytime need to print list item
    var props =
        entity.gridProperties; //.where((e) => e.dbName != 'active').toList();
    var dis = entity.gridProperties.where((e) => e.isDisplayField).toList();
    props.sort((a, b) => a.gridOrder - b.gridOrder);
    if (dis.length > 0) {
      dis.sort((a, b) => a.gridOrder - b.gridOrder);
      titleField = dis[0];
      props = props.where((e) => e.dbName != titleField?.dbName).toList();
      if (dis.length > 1)
        subTitleField = dis[1];
      else
        subTitleField = props.length > 0 ? props[0] : null;
      if (dis.length > 2)
        subTitleField = dis[2];
      else
        tailField = props.length > 1 ? props[1] : null;
      if (tailField == null && subTitleField != null) {
        tailField = subTitleField;
        subTitleField = null;
      }
    } else {
      titleField = props.length > 0 ? props[0] : null;
      subTitleField = props.length > 1 ? props[1] : null;
      tailField = props.length > 2 ? props[2] : null;
      if (tailField == null && subTitleField != null) {
        tailField = subTitleField;
        subTitleField = null;
      }
    }
  }

  @override
  State<EntityListTileComponent> createState() =>
      _EntityListTileComponentState();
}

class _EntityListTileComponentState extends AppState<EntityListTileComponent> {
  Widget _getTreeCell() {
    if ((widget.data['_children'] != null &&
            widget.data['_children'].length == 0) ||
        widget.onExpandItem == null)
      return SizedBox(
          width: 30 +
              (((widget.data['_tree_level'] ?? 0) as num).toDouble() * 10.0));
    return IconButton(
      tooltip: widget.data['_expanded'] == true ? 'Collapse' : 'Expand',
      padding: EdgeInsets.only(
          right: LocalConfigService.isArabic
              ? ((widget.data['_tree_level'] ?? 0) as num).toDouble() * 10.0
              : 0.0,
          left: LocalConfigService.isArabic
              ? 0.0
              : ((widget.data['_tree_level'] ?? 0) as num).toDouble() * 10.0),
      icon: widget.data['_expanded'] == true
          ? Icon(Icons.arrow_drop_down, size: 26)
          : RotatedBox(
              quarterTurns: 2,
              child: Icon(Icons.play_arrow_outlined, size: 20),
            ),
      onPressed: () => widget.onExpandItem?.call(widget.data),
    );
  }

  _onSaveNewItem(BuildContext context, newDataRaw) async {
    try {
      var validateResult =
          EntityService.validateEntity(widget.entity, newDataRaw);
      if (!validateResult.success) {
        AppNavigator.showMessage(
            context, validateResult.message ?? 'Error', MessageType.error);
        return;
      }
      var addResult = await EntityService.saveEntity(
          widget.entity, newDataRaw, context, false);
      if (addResult.success) {
        Navigator.pop(context, true); // close the add new item's form
      } else {
        AppNavigator.showMessage(
            context, addResult.message ?? '', MessageType.error);
      }
    } catch (err, trace) {
      print('XXXXXXXXX erorr adding item');
      print(err);
      print(trace);
      AppNavigator.showMessage(context, 'Error Adding Item', MessageType.error);
    }
  }

  Widget _getAddNewDialog(addDialogContext, newDataRaw) {
    return Container(
      constraints: BoxConstraints(maxHeight: 500, maxWidth: 500),
      child: Center(
        child: Card(
          child: ListView(
            children: <Widget>[
              InLineForm(widget.entity.dbName, true,
                  entity: widget.entity,
                  inPopup: true,
                  data: newDataRaw,
                  onChanged: (v) {}),
              Padding(
                  padding: EdgeInsets.symmetric(horizontal: 20),
                  child: Row(
                    children: <Widget>[
                      Expanded(child: Container()),
                      Expanded(
                          child: AppButtonPost(
                              label: 'Save',
                              onPressed: () => _onSaveNewItem(
                                  addDialogContext, newDataRaw))),
                      SizedBox(width: 10),
                      Expanded(
                          child: Container(
                              height: 50.0,
                              child: TextButton(
                                  child: Text('Cancel',
                                      style: ThemeService.flatButtonTextStyle),
                                  onPressed: () =>
                                      Navigator.pop(addDialogContext, false)))),
                      Expanded(child: Container()),
                    ],
                  )),
            ],
          ),
        ),
      ),
    );
  }

  Widget _editButton() {
    return IconButton(
        onPressed: () async {
          // Map<String, dynamic> newDataRaw =
          //     new Map<String, dynamic>();
          var addNewResult = await AppNavigator.showAppDialogEx(
              context, () => _getAddNewDialog(context, widget.data),
              size: DialogBoxSizes.large);
          if (addNewResult ?? false)
            Navigator.pop(context); // close the search form
        },
        tooltip: trans('Edit'),
        icon: Icon(Icons.note));
  }

  @override
  Widget build(BuildContext context) {
    return ListTile(
      leading: widget.entity.isTree && widget.onExpandItem != null
          ? _getTreeCell()
          : widget.entity.isTree
              ? Container(
                  width: 115,
                  child: Row(
                    mainAxisAlignment: MainAxisAlignment.spaceBetween,
                    children: [
                      IconButton(
                          onPressed: () => widget.onClick?.call(widget.data),
                          tooltip: trans('Subcategory'),
                          icon: Icon(Icons.subdirectory_arrow_left)),
                      widget.entity.hasPhoto
                          ? PhotoViewProperty(
                              data: widget.data[widget.entity.photoPropName]
                                  .toString())
                          : Container(),
                    ],
                  ))
              : widget.entity.hasPhoto
                  ? PhotoViewProperty(
                      data: widget.data[widget.entity.photoPropName].toString())
                  : null,
      title: widget.titleField == null
          ? null
          : Padding(
              padding: EdgeInsets.only(
                  right: LocalConfigService.isArabic
                      ? ((widget.data['_tree_level'] ?? 0) as num).toDouble() *
                          10.0
                      : 0.0,
                  left: LocalConfigService.isArabic
                      ? 0.0
                      : ((widget.data['_tree_level'] ?? 0) as num).toDouble() *
                          10.0),
              child: widget.titleField!.getDisplayWidget(widget.data,
                  textStyle: TextStyle(
                      fontSize: 16,
                      color: Colors
                          .black))), //, style:TextStyle(color: data['active'] == false ? Colors.red : Colors.black)),
      subtitle: widget.subTitleField == null
          ? null
          : Padding(
              padding: EdgeInsets.only(
                  right: LocalConfigService.isArabic
                      ? ((widget.data['_tree_level'] ?? 0) as num).toDouble() *
                          10
                      : 0,
                  left: LocalConfigService.isArabic
                      ? 0
                      : ((widget.data['_tree_level'] ?? 0) as num).toDouble() *
                          10),
              child: widget.subTitleField!.getDisplayWidget(widget.data,
                  textStyle: TextStyle(fontSize: 14, color: Colors.black54))),
      trailing: widget.tailField == null ? null : _editButton(),
      // tailField!.getDisplayWidget(data, textStyle: TextStyle(
      //             fontSize: 10, color: Colors.black87)),
      onTap: () => widget.onClick == null
          ? print('')
          : widget.entity.isTree
              ? widget.onLongClick?.call(widget.data)
              : widget.onClick?.call(widget.data),
      // onLongPress: () => widget.onLongClick?.call(widget.data)
    );
  }
}
